# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _dtv_swig2
else:
    import _dtv_swig2

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def high_res_timer_now() -> "gr::high_res_timer_type":
    r"""high_res_timer_now() -> gr::high_res_timer_type"""
    return _dtv_swig2.high_res_timer_now()

def high_res_timer_now_perfmon() -> "gr::high_res_timer_type":
    r"""high_res_timer_now_perfmon() -> gr::high_res_timer_type"""
    return _dtv_swig2.high_res_timer_now_perfmon()

def high_res_timer_tps() -> "gr::high_res_timer_type":
    r"""high_res_timer_tps() -> gr::high_res_timer_type"""
    return _dtv_swig2.high_res_timer_tps()

def high_res_timer_epoch() -> "gr::high_res_timer_type":
    r"""high_res_timer_epoch() -> gr::high_res_timer_type"""
    return _dtv_swig2.high_res_timer_epoch()
STANDARD_DVBS2 = _dtv_swig2.STANDARD_DVBS2

STANDARD_DVBT2 = _dtv_swig2.STANDARD_DVBT2

C1_4 = _dtv_swig2.C1_4

C1_3 = _dtv_swig2.C1_3

C2_5 = _dtv_swig2.C2_5

C1_2 = _dtv_swig2.C1_2

C3_5 = _dtv_swig2.C3_5

C2_3 = _dtv_swig2.C2_3

C3_4 = _dtv_swig2.C3_4

C4_5 = _dtv_swig2.C4_5

C5_6 = _dtv_swig2.C5_6

C7_8 = _dtv_swig2.C7_8

C8_9 = _dtv_swig2.C8_9

C9_10 = _dtv_swig2.C9_10

C13_45 = _dtv_swig2.C13_45

C9_20 = _dtv_swig2.C9_20

C90_180 = _dtv_swig2.C90_180

C96_180 = _dtv_swig2.C96_180

C11_20 = _dtv_swig2.C11_20

C100_180 = _dtv_swig2.C100_180

C104_180 = _dtv_swig2.C104_180

C26_45 = _dtv_swig2.C26_45

C18_30 = _dtv_swig2.C18_30

C28_45 = _dtv_swig2.C28_45

C23_36 = _dtv_swig2.C23_36

C116_180 = _dtv_swig2.C116_180

C20_30 = _dtv_swig2.C20_30

C124_180 = _dtv_swig2.C124_180

C25_36 = _dtv_swig2.C25_36

C128_180 = _dtv_swig2.C128_180

C13_18 = _dtv_swig2.C13_18

C132_180 = _dtv_swig2.C132_180

C22_30 = _dtv_swig2.C22_30

C135_180 = _dtv_swig2.C135_180

C140_180 = _dtv_swig2.C140_180

C7_9 = _dtv_swig2.C7_9

C154_180 = _dtv_swig2.C154_180

C11_45 = _dtv_swig2.C11_45

C4_15 = _dtv_swig2.C4_15

C14_45 = _dtv_swig2.C14_45

C7_15 = _dtv_swig2.C7_15

C8_15 = _dtv_swig2.C8_15

C32_45 = _dtv_swig2.C32_45

C2_9_VLSNR = _dtv_swig2.C2_9_VLSNR

C1_5_MEDIUM = _dtv_swig2.C1_5_MEDIUM

C11_45_MEDIUM = _dtv_swig2.C11_45_MEDIUM

C1_3_MEDIUM = _dtv_swig2.C1_3_MEDIUM

C1_5_VLSNR_SF2 = _dtv_swig2.C1_5_VLSNR_SF2

C11_45_VLSNR_SF2 = _dtv_swig2.C11_45_VLSNR_SF2

C1_5_VLSNR = _dtv_swig2.C1_5_VLSNR

C4_15_VLSNR = _dtv_swig2.C4_15_VLSNR

C1_3_VLSNR = _dtv_swig2.C1_3_VLSNR

C_OTHER = _dtv_swig2.C_OTHER

FECFRAME_SHORT = _dtv_swig2.FECFRAME_SHORT

FECFRAME_NORMAL = _dtv_swig2.FECFRAME_NORMAL

FECFRAME_MEDIUM = _dtv_swig2.FECFRAME_MEDIUM

MOD_QPSK = _dtv_swig2.MOD_QPSK

MOD_16QAM = _dtv_swig2.MOD_16QAM

MOD_64QAM = _dtv_swig2.MOD_64QAM

MOD_256QAM = _dtv_swig2.MOD_256QAM

MOD_8PSK = _dtv_swig2.MOD_8PSK

MOD_8APSK = _dtv_swig2.MOD_8APSK

MOD_16APSK = _dtv_swig2.MOD_16APSK

MOD_8_8APSK = _dtv_swig2.MOD_8_8APSK

MOD_32APSK = _dtv_swig2.MOD_32APSK

MOD_4_12_16APSK = _dtv_swig2.MOD_4_12_16APSK

MOD_4_8_4_16APSK = _dtv_swig2.MOD_4_8_4_16APSK

MOD_64APSK = _dtv_swig2.MOD_64APSK

MOD_8_16_20_20APSK = _dtv_swig2.MOD_8_16_20_20APSK

MOD_4_12_20_28APSK = _dtv_swig2.MOD_4_12_20_28APSK

MOD_128APSK = _dtv_swig2.MOD_128APSK

MOD_256APSK = _dtv_swig2.MOD_256APSK

MOD_BPSK = _dtv_swig2.MOD_BPSK

MOD_BPSK_SF2 = _dtv_swig2.MOD_BPSK_SF2

MOD_8VSB = _dtv_swig2.MOD_8VSB

MOD_OTHER = _dtv_swig2.MOD_OTHER

GI_1_32 = _dtv_swig2.GI_1_32

GI_1_16 = _dtv_swig2.GI_1_16

GI_1_8 = _dtv_swig2.GI_1_8

GI_1_4 = _dtv_swig2.GI_1_4

GI_1_128 = _dtv_swig2.GI_1_128

GI_19_128 = _dtv_swig2.GI_19_128

GI_19_256 = _dtv_swig2.GI_19_256

NH = _dtv_swig2.NH

ALPHA1 = _dtv_swig2.ALPHA1

ALPHA2 = _dtv_swig2.ALPHA2

ALPHA4 = _dtv_swig2.ALPHA4

T2k = _dtv_swig2.T2k

T8k = _dtv_swig2.T8k

class dvbt_energy_dispersal(object):
    r"""Proxy of C++ gr::dtv::dvbt_energy_dispersal class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nsize: "int") -> "gr::dtv::dvbt_energy_dispersal::sptr":
        r"""make(int nsize) -> dvbt_energy_dispersal_sptr"""
        return _dtv_swig2.dvbt_energy_dispersal_make(nsize)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_energy_dispersal

# Register dvbt_energy_dispersal in _dtv_swig2:
_dtv_swig2.dvbt_energy_dispersal_swigregister(dvbt_energy_dispersal)

def dvbt_energy_dispersal_make(nsize: "int") -> "gr::dtv::dvbt_energy_dispersal::sptr":
    r"""dvbt_energy_dispersal_make(int nsize) -> dvbt_energy_dispersal_sptr"""
    return _dtv_swig2.dvbt_energy_dispersal_make(nsize)

class dvbt_reed_solomon_enc(object):
    r"""Proxy of C++ gr::dtv::dvbt_reed_solomon_enc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(p: "int", m: "int", gfpoly: "int", n: "int", k: "int", t: "int", s: "int", blocks: "int") -> "gr::dtv::dvbt_reed_solomon_enc::sptr":
        r"""make(int p, int m, int gfpoly, int n, int k, int t, int s, int blocks) -> dvbt_reed_solomon_enc_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_make(p, m, gfpoly, n, k, t, s, blocks)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_reed_solomon_enc

# Register dvbt_reed_solomon_enc in _dtv_swig2:
_dtv_swig2.dvbt_reed_solomon_enc_swigregister(dvbt_reed_solomon_enc)

def dvbt_reed_solomon_enc_make(p: "int", m: "int", gfpoly: "int", n: "int", k: "int", t: "int", s: "int", blocks: "int") -> "gr::dtv::dvbt_reed_solomon_enc::sptr":
    r"""dvbt_reed_solomon_enc_make(int p, int m, int gfpoly, int n, int k, int t, int s, int blocks) -> dvbt_reed_solomon_enc_sptr"""
    return _dtv_swig2.dvbt_reed_solomon_enc_make(p, m, gfpoly, n, k, t, s, blocks)

class dvbt_convolutional_interleaver(object):
    r"""Proxy of C++ gr::dtv::dvbt_convolutional_interleaver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nsize: "int", I: "int", M: "int") -> "gr::dtv::dvbt_convolutional_interleaver::sptr":
        r"""make(int nsize, int I, int M) -> dvbt_convolutional_interleaver_sptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_make(nsize, I, M)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_convolutional_interleaver

# Register dvbt_convolutional_interleaver in _dtv_swig2:
_dtv_swig2.dvbt_convolutional_interleaver_swigregister(dvbt_convolutional_interleaver)

def dvbt_convolutional_interleaver_make(nsize: "int", I: "int", M: "int") -> "gr::dtv::dvbt_convolutional_interleaver::sptr":
    r"""dvbt_convolutional_interleaver_make(int nsize, int I, int M) -> dvbt_convolutional_interleaver_sptr"""
    return _dtv_swig2.dvbt_convolutional_interleaver_make(nsize, I, M)

class dvbt_inner_coder(object):
    r"""Proxy of C++ gr::dtv::dvbt_inner_coder class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", coderate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvbt_inner_coder::sptr":
        r"""make(int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t coderate) -> dvbt_inner_coder_sptr"""
        return _dtv_swig2.dvbt_inner_coder_make(ninput, noutput, constellation, hierarchy, coderate)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_inner_coder

# Register dvbt_inner_coder in _dtv_swig2:
_dtv_swig2.dvbt_inner_coder_swigregister(dvbt_inner_coder)

def dvbt_inner_coder_make(ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", coderate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvbt_inner_coder::sptr":
    r"""dvbt_inner_coder_make(int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t coderate) -> dvbt_inner_coder_sptr"""
    return _dtv_swig2.dvbt_inner_coder_make(ninput, noutput, constellation, hierarchy, coderate)

class dvbt_bit_inner_interleaver(object):
    r"""Proxy of C++ gr::dtv::dvbt_bit_inner_interleaver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t") -> "gr::dtv::dvbt_bit_inner_interleaver::sptr":
        r"""make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission) -> dvbt_bit_inner_interleaver_sptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_make(nsize, constellation, hierarchy, transmission)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_bit_inner_interleaver

# Register dvbt_bit_inner_interleaver in _dtv_swig2:
_dtv_swig2.dvbt_bit_inner_interleaver_swigregister(dvbt_bit_inner_interleaver)

def dvbt_bit_inner_interleaver_make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t") -> "gr::dtv::dvbt_bit_inner_interleaver::sptr":
    r"""dvbt_bit_inner_interleaver_make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission) -> dvbt_bit_inner_interleaver_sptr"""
    return _dtv_swig2.dvbt_bit_inner_interleaver_make(nsize, constellation, hierarchy, transmission)

class dvbt_symbol_inner_interleaver(object):
    r"""Proxy of C++ gr::dtv::dvbt_symbol_inner_interleaver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(ninput: "int", transmission: "gr::dtv::dvbt_transmission_mode_t", direction: "int") -> "gr::dtv::dvbt_symbol_inner_interleaver::sptr":
        r"""make(int ninput, gr::dtv::dvbt_transmission_mode_t transmission, int direction) -> dvbt_symbol_inner_interleaver_sptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_make(ninput, transmission, direction)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_symbol_inner_interleaver

# Register dvbt_symbol_inner_interleaver in _dtv_swig2:
_dtv_swig2.dvbt_symbol_inner_interleaver_swigregister(dvbt_symbol_inner_interleaver)

def dvbt_symbol_inner_interleaver_make(ninput: "int", transmission: "gr::dtv::dvbt_transmission_mode_t", direction: "int") -> "gr::dtv::dvbt_symbol_inner_interleaver::sptr":
    r"""dvbt_symbol_inner_interleaver_make(int ninput, gr::dtv::dvbt_transmission_mode_t transmission, int direction) -> dvbt_symbol_inner_interleaver_sptr"""
    return _dtv_swig2.dvbt_symbol_inner_interleaver_make(ninput, transmission, direction)

class dvbt_map(object):
    r"""Proxy of C++ gr::dtv::dvbt_map class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t", gain: "float") -> "gr::dtv::dvbt_map::sptr":
        r"""make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission, float gain) -> dvbt_map_sptr"""
        return _dtv_swig2.dvbt_map_make(nsize, constellation, hierarchy, transmission, gain)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_map

# Register dvbt_map in _dtv_swig2:
_dtv_swig2.dvbt_map_swigregister(dvbt_map)

def dvbt_map_make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t", gain: "float") -> "gr::dtv::dvbt_map::sptr":
    r"""dvbt_map_make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission, float gain) -> dvbt_map_sptr"""
    return _dtv_swig2.dvbt_map_make(nsize, constellation, hierarchy, transmission, gain)

class dvbt_reference_signals(object):
    r"""Proxy of C++ gr::dtv::dvbt_reference_signals class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(itemsize: "int", ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", code_rate_HP: "gr::dtv::dvb_code_rate_t", code_rate_LP: "gr::dtv::dvb_code_rate_t", guard_interval: "gr::dtv::dvb_guardinterval_t", transmission_mode: "gr::dtv::dvbt_transmission_mode_t", include_cell_id: "int", cell_id: "int") -> "gr::dtv::dvbt_reference_signals::sptr":
        r"""make(int itemsize, int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t code_rate_HP, gr::dtv::dvb_code_rate_t code_rate_LP, gr::dtv::dvb_guardinterval_t guard_interval, gr::dtv::dvbt_transmission_mode_t transmission_mode, int include_cell_id, int cell_id) -> dvbt_reference_signals_sptr"""
        return _dtv_swig2.dvbt_reference_signals_make(itemsize, ninput, noutput, constellation, hierarchy, code_rate_HP, code_rate_LP, guard_interval, transmission_mode, include_cell_id, cell_id)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_reference_signals

# Register dvbt_reference_signals in _dtv_swig2:
_dtv_swig2.dvbt_reference_signals_swigregister(dvbt_reference_signals)

def dvbt_reference_signals_make(itemsize: "int", ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", code_rate_HP: "gr::dtv::dvb_code_rate_t", code_rate_LP: "gr::dtv::dvb_code_rate_t", guard_interval: "gr::dtv::dvb_guardinterval_t", transmission_mode: "gr::dtv::dvbt_transmission_mode_t", include_cell_id: "int", cell_id: "int") -> "gr::dtv::dvbt_reference_signals::sptr":
    r"""dvbt_reference_signals_make(int itemsize, int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t code_rate_HP, gr::dtv::dvb_code_rate_t code_rate_LP, gr::dtv::dvb_guardinterval_t guard_interval, gr::dtv::dvbt_transmission_mode_t transmission_mode, int include_cell_id, int cell_id) -> dvbt_reference_signals_sptr"""
    return _dtv_swig2.dvbt_reference_signals_make(itemsize, ninput, noutput, constellation, hierarchy, code_rate_HP, code_rate_LP, guard_interval, transmission_mode, include_cell_id, cell_id)

class dvbt_ofdm_sym_acquisition(object):
    r"""Proxy of C++ gr::dtv::dvbt_ofdm_sym_acquisition class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(blocks: "int", fft_length: "int", occupied_tones: "int", cp_length: "int", snr: "float") -> "gr::dtv::dvbt_ofdm_sym_acquisition::sptr":
        r"""make(int blocks, int fft_length, int occupied_tones, int cp_length, float snr) -> dvbt_ofdm_sym_acquisition_sptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_make(blocks, fft_length, occupied_tones, cp_length, snr)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_ofdm_sym_acquisition

# Register dvbt_ofdm_sym_acquisition in _dtv_swig2:
_dtv_swig2.dvbt_ofdm_sym_acquisition_swigregister(dvbt_ofdm_sym_acquisition)

def dvbt_ofdm_sym_acquisition_make(blocks: "int", fft_length: "int", occupied_tones: "int", cp_length: "int", snr: "float") -> "gr::dtv::dvbt_ofdm_sym_acquisition::sptr":
    r"""dvbt_ofdm_sym_acquisition_make(int blocks, int fft_length, int occupied_tones, int cp_length, float snr) -> dvbt_ofdm_sym_acquisition_sptr"""
    return _dtv_swig2.dvbt_ofdm_sym_acquisition_make(blocks, fft_length, occupied_tones, cp_length, snr)

class dvbt_demod_reference_signals(object):
    r"""Proxy of C++ gr::dtv::dvbt_demod_reference_signals class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(itemsize: "int", ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", code_rate_HP: "gr::dtv::dvb_code_rate_t", code_rate_LP: "gr::dtv::dvb_code_rate_t", guard_interval: "gr::dtv::dvb_guardinterval_t", transmission_mode: "gr::dtv::dvbt_transmission_mode_t", include_cell_id: "int", cell_id: "int") -> "gr::dtv::dvbt_demod_reference_signals::sptr":
        r"""make(int itemsize, int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t code_rate_HP, gr::dtv::dvb_code_rate_t code_rate_LP, gr::dtv::dvb_guardinterval_t guard_interval, gr::dtv::dvbt_transmission_mode_t transmission_mode, int include_cell_id, int cell_id) -> dvbt_demod_reference_signals_sptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_make(itemsize, ninput, noutput, constellation, hierarchy, code_rate_HP, code_rate_LP, guard_interval, transmission_mode, include_cell_id, cell_id)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_demod_reference_signals

# Register dvbt_demod_reference_signals in _dtv_swig2:
_dtv_swig2.dvbt_demod_reference_signals_swigregister(dvbt_demod_reference_signals)

def dvbt_demod_reference_signals_make(itemsize: "int", ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", code_rate_HP: "gr::dtv::dvb_code_rate_t", code_rate_LP: "gr::dtv::dvb_code_rate_t", guard_interval: "gr::dtv::dvb_guardinterval_t", transmission_mode: "gr::dtv::dvbt_transmission_mode_t", include_cell_id: "int", cell_id: "int") -> "gr::dtv::dvbt_demod_reference_signals::sptr":
    r"""dvbt_demod_reference_signals_make(int itemsize, int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t code_rate_HP, gr::dtv::dvb_code_rate_t code_rate_LP, gr::dtv::dvb_guardinterval_t guard_interval, gr::dtv::dvbt_transmission_mode_t transmission_mode, int include_cell_id, int cell_id) -> dvbt_demod_reference_signals_sptr"""
    return _dtv_swig2.dvbt_demod_reference_signals_make(itemsize, ninput, noutput, constellation, hierarchy, code_rate_HP, code_rate_LP, guard_interval, transmission_mode, include_cell_id, cell_id)

class dvbt_demap(object):
    r"""Proxy of C++ gr::dtv::dvbt_demap class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t", gain: "float") -> "gr::dtv::dvbt_demap::sptr":
        r"""make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission, float gain) -> dvbt_demap_sptr"""
        return _dtv_swig2.dvbt_demap_make(nsize, constellation, hierarchy, transmission, gain)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_demap

# Register dvbt_demap in _dtv_swig2:
_dtv_swig2.dvbt_demap_swigregister(dvbt_demap)

def dvbt_demap_make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t", gain: "float") -> "gr::dtv::dvbt_demap::sptr":
    r"""dvbt_demap_make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission, float gain) -> dvbt_demap_sptr"""
    return _dtv_swig2.dvbt_demap_make(nsize, constellation, hierarchy, transmission, gain)

class dvbt_bit_inner_deinterleaver(object):
    r"""Proxy of C++ gr::dtv::dvbt_bit_inner_deinterleaver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t") -> "gr::dtv::dvbt_bit_inner_deinterleaver::sptr":
        r"""make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission) -> dvbt_bit_inner_deinterleaver_sptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_make(nsize, constellation, hierarchy, transmission)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_bit_inner_deinterleaver

# Register dvbt_bit_inner_deinterleaver in _dtv_swig2:
_dtv_swig2.dvbt_bit_inner_deinterleaver_swigregister(dvbt_bit_inner_deinterleaver)

def dvbt_bit_inner_deinterleaver_make(nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t") -> "gr::dtv::dvbt_bit_inner_deinterleaver::sptr":
    r"""dvbt_bit_inner_deinterleaver_make(int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission) -> dvbt_bit_inner_deinterleaver_sptr"""
    return _dtv_swig2.dvbt_bit_inner_deinterleaver_make(nsize, constellation, hierarchy, transmission)

class dvbt_viterbi_decoder(object):
    r"""Proxy of C++ gr::dtv::dvbt_viterbi_decoder class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", coderate: "gr::dtv::dvb_code_rate_t", bsize: "int") -> "gr::dtv::dvbt_viterbi_decoder::sptr":
        r"""make(gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t coderate, int bsize) -> dvbt_viterbi_decoder_sptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_make(constellation, hierarchy, coderate, bsize)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_viterbi_decoder

# Register dvbt_viterbi_decoder in _dtv_swig2:
_dtv_swig2.dvbt_viterbi_decoder_swigregister(dvbt_viterbi_decoder)

def dvbt_viterbi_decoder_make(constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", coderate: "gr::dtv::dvb_code_rate_t", bsize: "int") -> "gr::dtv::dvbt_viterbi_decoder::sptr":
    r"""dvbt_viterbi_decoder_make(gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t coderate, int bsize) -> dvbt_viterbi_decoder_sptr"""
    return _dtv_swig2.dvbt_viterbi_decoder_make(constellation, hierarchy, coderate, bsize)

class dvbt_convolutional_deinterleaver(object):
    r"""Proxy of C++ gr::dtv::dvbt_convolutional_deinterleaver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nsize: "int", I: "int", M: "int") -> "gr::dtv::dvbt_convolutional_deinterleaver::sptr":
        r"""make(int nsize, int I, int M) -> dvbt_convolutional_deinterleaver_sptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_make(nsize, I, M)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_convolutional_deinterleaver

# Register dvbt_convolutional_deinterleaver in _dtv_swig2:
_dtv_swig2.dvbt_convolutional_deinterleaver_swigregister(dvbt_convolutional_deinterleaver)

def dvbt_convolutional_deinterleaver_make(nsize: "int", I: "int", M: "int") -> "gr::dtv::dvbt_convolutional_deinterleaver::sptr":
    r"""dvbt_convolutional_deinterleaver_make(int nsize, int I, int M) -> dvbt_convolutional_deinterleaver_sptr"""
    return _dtv_swig2.dvbt_convolutional_deinterleaver_make(nsize, I, M)

class dvbt_reed_solomon_dec(object):
    r"""Proxy of C++ gr::dtv::dvbt_reed_solomon_dec class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(p: "int", m: "int", gfpoly: "int", n: "int", k: "int", t: "int", s: "int", blocks: "int") -> "gr::dtv::dvbt_reed_solomon_dec::sptr":
        r"""make(int p, int m, int gfpoly, int n, int k, int t, int s, int blocks) -> dvbt_reed_solomon_dec_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_make(p, m, gfpoly, n, k, t, s, blocks)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_reed_solomon_dec

# Register dvbt_reed_solomon_dec in _dtv_swig2:
_dtv_swig2.dvbt_reed_solomon_dec_swigregister(dvbt_reed_solomon_dec)

def dvbt_reed_solomon_dec_make(p: "int", m: "int", gfpoly: "int", n: "int", k: "int", t: "int", s: "int", blocks: "int") -> "gr::dtv::dvbt_reed_solomon_dec::sptr":
    r"""dvbt_reed_solomon_dec_make(int p, int m, int gfpoly, int n, int k, int t, int s, int blocks) -> dvbt_reed_solomon_dec_sptr"""
    return _dtv_swig2.dvbt_reed_solomon_dec_make(p, m, gfpoly, n, k, t, s, blocks)

class dvbt_energy_descramble(object):
    r"""Proxy of C++ gr::dtv::dvbt_energy_descramble class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(nblocks: "int") -> "gr::dtv::dvbt_energy_descramble::sptr":
        r"""make(int nblocks) -> dvbt_energy_descramble_sptr"""
        return _dtv_swig2.dvbt_energy_descramble_make(nblocks)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_energy_descramble

# Register dvbt_energy_descramble in _dtv_swig2:
_dtv_swig2.dvbt_energy_descramble_swigregister(dvbt_energy_descramble)

def dvbt_energy_descramble_make(nblocks: "int") -> "gr::dtv::dvbt_energy_descramble::sptr":
    r"""dvbt_energy_descramble_make(int nblocks) -> dvbt_energy_descramble_sptr"""
    return _dtv_swig2.dvbt_energy_descramble_make(nblocks)

class dvbt_energy_dispersal_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_energy_dispersal > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_energy_dispersal_sptr self) -> dvbt_energy_dispersal_sptr
        __init__(dvbt_energy_dispersal_sptr self, dvbt_energy_dispersal p) -> dvbt_energy_dispersal_sptr
        """
        _dtv_swig2.dvbt_energy_dispersal_sptr_swiginit(self, _dtv_swig2.new_dvbt_energy_dispersal_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_energy_dispersal *":
        r"""__deref__(dvbt_energy_dispersal_sptr self) -> dvbt_energy_dispersal"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_energy_dispersal_sptr

    def make(self, nsize: "int") -> "gr::dtv::dvbt_energy_dispersal::sptr":
        r"""make(dvbt_energy_dispersal_sptr self, int nsize) -> dvbt_energy_dispersal_sptr"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_make(self, nsize)

    def history(self) -> "unsigned int":
        r"""history(dvbt_energy_dispersal_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_energy_dispersal_sptr self, int which, int delay)
        declare_sample_delay(dvbt_energy_dispersal_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_energy_dispersal_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_energy_dispersal_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_energy_dispersal_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_energy_dispersal_sptr self) -> double"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_energy_dispersal_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_energy_dispersal_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_energy_dispersal_sptr self) -> bool"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_energy_dispersal_sptr self) -> bool"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_energy_dispersal_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_energy_dispersal_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_energy_dispersal_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_energy_dispersal_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_energy_dispersal_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_energy_dispersal_sptr self, int m)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_energy_dispersal_sptr self)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_energy_dispersal_sptr self) -> bool"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_energy_dispersal_sptr self, int m)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_energy_dispersal_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_energy_dispersal_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_energy_dispersal_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_energy_dispersal_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_energy_dispersal_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_energy_dispersal_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_energy_dispersal_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_energy_dispersal_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_energy_dispersal_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_energy_dispersal_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_energy_dispersal_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_energy_dispersal_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_energy_dispersal_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_energy_dispersal_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_energy_dispersal_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_energy_dispersal_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_energy_dispersal_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_energy_dispersal_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_energy_dispersal_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_energy_dispersal_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_energy_dispersal_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_energy_dispersal_sptr self)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_energy_dispersal_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_energy_dispersal_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_energy_dispersal_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_energy_dispersal_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_energy_dispersal_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_energy_dispersal_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_energy_dispersal_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_energy_dispersal_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_energy_dispersal_sptr self) -> long"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_energy_dispersal_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_energy_dispersal_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_energy_dispersal_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_energy_dispersal_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_energy_dispersal_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_energy_dispersal_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_energy_dispersal_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_energy_dispersal_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_energy_dispersal_sptr_message_subscribers(self, which_port)

# Register dvbt_energy_dispersal_sptr in _dtv_swig2:
_dtv_swig2.dvbt_energy_dispersal_sptr_swigregister(dvbt_energy_dispersal_sptr)


dvbt_energy_dispersal_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_energy_dispersal = dvbt_energy_dispersal.make;

class dvbt_reed_solomon_enc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_reed_solomon_enc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_reed_solomon_enc_sptr self) -> dvbt_reed_solomon_enc_sptr
        __init__(dvbt_reed_solomon_enc_sptr self, dvbt_reed_solomon_enc p) -> dvbt_reed_solomon_enc_sptr
        """
        _dtv_swig2.dvbt_reed_solomon_enc_sptr_swiginit(self, _dtv_swig2.new_dvbt_reed_solomon_enc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_reed_solomon_enc *":
        r"""__deref__(dvbt_reed_solomon_enc_sptr self) -> dvbt_reed_solomon_enc"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_reed_solomon_enc_sptr

    def make(self, p: "int", m: "int", gfpoly: "int", n: "int", k: "int", t: "int", s: "int", blocks: "int") -> "gr::dtv::dvbt_reed_solomon_enc::sptr":
        r"""make(dvbt_reed_solomon_enc_sptr self, int p, int m, int gfpoly, int n, int k, int t, int s, int blocks) -> dvbt_reed_solomon_enc_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_make(self, p, m, gfpoly, n, k, t, s, blocks)

    def history(self) -> "unsigned int":
        r"""history(dvbt_reed_solomon_enc_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_reed_solomon_enc_sptr self, int which, int delay)
        declare_sample_delay(dvbt_reed_solomon_enc_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_reed_solomon_enc_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_reed_solomon_enc_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_reed_solomon_enc_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_reed_solomon_enc_sptr self) -> double"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_reed_solomon_enc_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_reed_solomon_enc_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_reed_solomon_enc_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_reed_solomon_enc_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_reed_solomon_enc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_reed_solomon_enc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_reed_solomon_enc_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_reed_solomon_enc_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_reed_solomon_enc_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_reed_solomon_enc_sptr self, int m)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_reed_solomon_enc_sptr self)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_reed_solomon_enc_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_reed_solomon_enc_sptr self, int m)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_reed_solomon_enc_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_reed_solomon_enc_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_reed_solomon_enc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_reed_solomon_enc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_reed_solomon_enc_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_reed_solomon_enc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_reed_solomon_enc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_reed_solomon_enc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_reed_solomon_enc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_reed_solomon_enc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_reed_solomon_enc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_reed_solomon_enc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_reed_solomon_enc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_reed_solomon_enc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_reed_solomon_enc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_reed_solomon_enc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_reed_solomon_enc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_reed_solomon_enc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_reed_solomon_enc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_reed_solomon_enc_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_reed_solomon_enc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_reed_solomon_enc_sptr self)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_reed_solomon_enc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_reed_solomon_enc_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_reed_solomon_enc_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_reed_solomon_enc_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_reed_solomon_enc_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_reed_solomon_enc_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_reed_solomon_enc_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_reed_solomon_enc_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_reed_solomon_enc_sptr self) -> long"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_reed_solomon_enc_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_reed_solomon_enc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_reed_solomon_enc_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_reed_solomon_enc_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_reed_solomon_enc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_reed_solomon_enc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_reed_solomon_enc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_reed_solomon_enc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reed_solomon_enc_sptr_message_subscribers(self, which_port)

# Register dvbt_reed_solomon_enc_sptr in _dtv_swig2:
_dtv_swig2.dvbt_reed_solomon_enc_sptr_swigregister(dvbt_reed_solomon_enc_sptr)


dvbt_reed_solomon_enc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_reed_solomon_enc = dvbt_reed_solomon_enc.make;

class dvbt_convolutional_interleaver_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_convolutional_interleaver > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_convolutional_interleaver_sptr self) -> dvbt_convolutional_interleaver_sptr
        __init__(dvbt_convolutional_interleaver_sptr self, dvbt_convolutional_interleaver p) -> dvbt_convolutional_interleaver_sptr
        """
        _dtv_swig2.dvbt_convolutional_interleaver_sptr_swiginit(self, _dtv_swig2.new_dvbt_convolutional_interleaver_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_convolutional_interleaver *":
        r"""__deref__(dvbt_convolutional_interleaver_sptr self) -> dvbt_convolutional_interleaver"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_convolutional_interleaver_sptr

    def make(self, nsize: "int", I: "int", M: "int") -> "gr::dtv::dvbt_convolutional_interleaver::sptr":
        r"""make(dvbt_convolutional_interleaver_sptr self, int nsize, int I, int M) -> dvbt_convolutional_interleaver_sptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_make(self, nsize, I, M)

    def history(self) -> "unsigned int":
        r"""history(dvbt_convolutional_interleaver_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_convolutional_interleaver_sptr self, int which, int delay)
        declare_sample_delay(dvbt_convolutional_interleaver_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_convolutional_interleaver_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_convolutional_interleaver_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_convolutional_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_convolutional_interleaver_sptr self) -> double"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_convolutional_interleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_convolutional_interleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_convolutional_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_convolutional_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_convolutional_interleaver_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_convolutional_interleaver_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_convolutional_interleaver_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_convolutional_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_convolutional_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_convolutional_interleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_convolutional_interleaver_sptr self)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_convolutional_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_convolutional_interleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_convolutional_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_convolutional_interleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_convolutional_interleaver_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_convolutional_interleaver_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_convolutional_interleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_convolutional_interleaver_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_convolutional_interleaver_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_convolutional_interleaver_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_convolutional_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_convolutional_interleaver_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_convolutional_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_convolutional_interleaver_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_convolutional_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_convolutional_interleaver_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_convolutional_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_convolutional_interleaver_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_convolutional_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_convolutional_interleaver_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_convolutional_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_convolutional_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_convolutional_interleaver_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_convolutional_interleaver_sptr self)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_convolutional_interleaver_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_convolutional_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_convolutional_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_convolutional_interleaver_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_convolutional_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_convolutional_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_convolutional_interleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_convolutional_interleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_convolutional_interleaver_sptr self) -> long"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_convolutional_interleaver_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_convolutional_interleaver_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_convolutional_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_convolutional_interleaver_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_convolutional_interleaver_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_convolutional_interleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_convolutional_interleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_convolutional_interleaver_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_convolutional_interleaver_sptr_message_subscribers(self, which_port)

# Register dvbt_convolutional_interleaver_sptr in _dtv_swig2:
_dtv_swig2.dvbt_convolutional_interleaver_sptr_swigregister(dvbt_convolutional_interleaver_sptr)


dvbt_convolutional_interleaver_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_convolutional_interleaver = dvbt_convolutional_interleaver.make;

class dvbt_inner_coder_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_inner_coder > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_inner_coder_sptr self) -> dvbt_inner_coder_sptr
        __init__(dvbt_inner_coder_sptr self, dvbt_inner_coder p) -> dvbt_inner_coder_sptr
        """
        _dtv_swig2.dvbt_inner_coder_sptr_swiginit(self, _dtv_swig2.new_dvbt_inner_coder_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_inner_coder *":
        r"""__deref__(dvbt_inner_coder_sptr self) -> dvbt_inner_coder"""
        return _dtv_swig2.dvbt_inner_coder_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_inner_coder_sptr

    def make(self, ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", coderate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvbt_inner_coder::sptr":
        r"""make(dvbt_inner_coder_sptr self, int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t coderate) -> dvbt_inner_coder_sptr"""
        return _dtv_swig2.dvbt_inner_coder_sptr_make(self, ninput, noutput, constellation, hierarchy, coderate)

    def history(self) -> "unsigned int":
        r"""history(dvbt_inner_coder_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_inner_coder_sptr self, int which, int delay)
        declare_sample_delay(dvbt_inner_coder_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_inner_coder_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_inner_coder_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_inner_coder_sptr self) -> int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_inner_coder_sptr self) -> double"""
        return _dtv_swig2.dvbt_inner_coder_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_inner_coder_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_inner_coder_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_inner_coder_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_inner_coder_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_inner_coder_sptr self) -> bool"""
        return _dtv_swig2.dvbt_inner_coder_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_inner_coder_sptr self) -> bool"""
        return _dtv_swig2.dvbt_inner_coder_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_inner_coder_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_inner_coder_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_inner_coder_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_inner_coder_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_inner_coder_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_inner_coder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_inner_coder_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_inner_coder_sptr self) -> int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_inner_coder_sptr self, int m)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_inner_coder_sptr self)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_inner_coder_sptr self) -> bool"""
        return _dtv_swig2.dvbt_inner_coder_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_inner_coder_sptr self, int m)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_inner_coder_sptr self) -> int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_inner_coder_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_inner_coder_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_inner_coder_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_inner_coder_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_inner_coder_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_inner_coder_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_inner_coder_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_inner_coder_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_inner_coder_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_inner_coder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_inner_coder_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_inner_coder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_inner_coder_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_inner_coder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_inner_coder_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_inner_coder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_inner_coder_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_inner_coder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_inner_coder_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_inner_coder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_inner_coder_sptr self) -> float"""
        return _dtv_swig2.dvbt_inner_coder_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_inner_coder_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_inner_coder_sptr self)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_inner_coder_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_inner_coder_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_inner_coder_sptr self) -> int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_inner_coder_sptr self) -> int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_inner_coder_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_inner_coder_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_inner_coder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_inner_coder_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_inner_coder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_inner_coder_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_inner_coder_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_inner_coder_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_inner_coder_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_inner_coder_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_inner_coder_sptr self) -> long"""
        return _dtv_swig2.dvbt_inner_coder_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_inner_coder_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_inner_coder_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_inner_coder_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_inner_coder_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_inner_coder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_inner_coder_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_inner_coder_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_inner_coder_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_inner_coder_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_inner_coder_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_inner_coder_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_inner_coder_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_inner_coder_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_inner_coder_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_inner_coder_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_inner_coder_sptr_message_subscribers(self, which_port)

# Register dvbt_inner_coder_sptr in _dtv_swig2:
_dtv_swig2.dvbt_inner_coder_sptr_swigregister(dvbt_inner_coder_sptr)


dvbt_inner_coder_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_inner_coder = dvbt_inner_coder.make;

class dvbt_bit_inner_interleaver_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_bit_inner_interleaver > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_bit_inner_interleaver_sptr self) -> dvbt_bit_inner_interleaver_sptr
        __init__(dvbt_bit_inner_interleaver_sptr self, dvbt_bit_inner_interleaver p) -> dvbt_bit_inner_interleaver_sptr
        """
        _dtv_swig2.dvbt_bit_inner_interleaver_sptr_swiginit(self, _dtv_swig2.new_dvbt_bit_inner_interleaver_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_bit_inner_interleaver *":
        r"""__deref__(dvbt_bit_inner_interleaver_sptr self) -> dvbt_bit_inner_interleaver"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_bit_inner_interleaver_sptr

    def make(self, nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t") -> "gr::dtv::dvbt_bit_inner_interleaver::sptr":
        r"""make(dvbt_bit_inner_interleaver_sptr self, int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission) -> dvbt_bit_inner_interleaver_sptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_make(self, nsize, constellation, hierarchy, transmission)

    def history(self) -> "unsigned int":
        r"""history(dvbt_bit_inner_interleaver_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_bit_inner_interleaver_sptr self, int which, int delay)
        declare_sample_delay(dvbt_bit_inner_interleaver_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_bit_inner_interleaver_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_bit_inner_interleaver_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_bit_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_bit_inner_interleaver_sptr self) -> double"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_bit_inner_interleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_bit_inner_interleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_bit_inner_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_bit_inner_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_bit_inner_interleaver_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_bit_inner_interleaver_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_bit_inner_interleaver_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_bit_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_bit_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_bit_inner_interleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_bit_inner_interleaver_sptr self)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_bit_inner_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_bit_inner_interleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_bit_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_bit_inner_interleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_bit_inner_interleaver_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_bit_inner_interleaver_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_bit_inner_interleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_bit_inner_interleaver_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_bit_inner_interleaver_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_bit_inner_interleaver_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_bit_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_bit_inner_interleaver_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_bit_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_bit_inner_interleaver_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_bit_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_bit_inner_interleaver_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_bit_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_bit_inner_interleaver_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_bit_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_bit_inner_interleaver_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_bit_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_bit_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_bit_inner_interleaver_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_bit_inner_interleaver_sptr self)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_bit_inner_interleaver_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_bit_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_bit_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_bit_inner_interleaver_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_bit_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_bit_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_bit_inner_interleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_bit_inner_interleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_bit_inner_interleaver_sptr self) -> long"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_bit_inner_interleaver_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_bit_inner_interleaver_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_bit_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_bit_inner_interleaver_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_bit_inner_interleaver_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_bit_inner_interleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_bit_inner_interleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_bit_inner_interleaver_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_bit_inner_interleaver_sptr_message_subscribers(self, which_port)

# Register dvbt_bit_inner_interleaver_sptr in _dtv_swig2:
_dtv_swig2.dvbt_bit_inner_interleaver_sptr_swigregister(dvbt_bit_inner_interleaver_sptr)


dvbt_bit_inner_interleaver_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_bit_inner_interleaver = dvbt_bit_inner_interleaver.make;

class dvbt_symbol_inner_interleaver_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_symbol_inner_interleaver > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_symbol_inner_interleaver_sptr self) -> dvbt_symbol_inner_interleaver_sptr
        __init__(dvbt_symbol_inner_interleaver_sptr self, dvbt_symbol_inner_interleaver p) -> dvbt_symbol_inner_interleaver_sptr
        """
        _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_swiginit(self, _dtv_swig2.new_dvbt_symbol_inner_interleaver_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_symbol_inner_interleaver *":
        r"""__deref__(dvbt_symbol_inner_interleaver_sptr self) -> dvbt_symbol_inner_interleaver"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_symbol_inner_interleaver_sptr

    def make(self, ninput: "int", transmission: "gr::dtv::dvbt_transmission_mode_t", direction: "int") -> "gr::dtv::dvbt_symbol_inner_interleaver::sptr":
        r"""make(dvbt_symbol_inner_interleaver_sptr self, int ninput, gr::dtv::dvbt_transmission_mode_t transmission, int direction) -> dvbt_symbol_inner_interleaver_sptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_make(self, ninput, transmission, direction)

    def history(self) -> "unsigned int":
        r"""history(dvbt_symbol_inner_interleaver_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_symbol_inner_interleaver_sptr self, int which, int delay)
        declare_sample_delay(dvbt_symbol_inner_interleaver_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_symbol_inner_interleaver_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_symbol_inner_interleaver_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_symbol_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_symbol_inner_interleaver_sptr self) -> double"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_symbol_inner_interleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_symbol_inner_interleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_symbol_inner_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_symbol_inner_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_symbol_inner_interleaver_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_symbol_inner_interleaver_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_symbol_inner_interleaver_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_symbol_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_symbol_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_symbol_inner_interleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_symbol_inner_interleaver_sptr self)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_symbol_inner_interleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_symbol_inner_interleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_symbol_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_symbol_inner_interleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_symbol_inner_interleaver_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_symbol_inner_interleaver_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_symbol_inner_interleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_symbol_inner_interleaver_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_symbol_inner_interleaver_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_symbol_inner_interleaver_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_symbol_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_symbol_inner_interleaver_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_symbol_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_symbol_inner_interleaver_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_symbol_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_symbol_inner_interleaver_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_symbol_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_symbol_inner_interleaver_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_symbol_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_symbol_inner_interleaver_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_symbol_inner_interleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_symbol_inner_interleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_symbol_inner_interleaver_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_symbol_inner_interleaver_sptr self)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_symbol_inner_interleaver_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_symbol_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_symbol_inner_interleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_symbol_inner_interleaver_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_symbol_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_symbol_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_symbol_inner_interleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_symbol_inner_interleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_symbol_inner_interleaver_sptr self) -> long"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_symbol_inner_interleaver_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_symbol_inner_interleaver_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_symbol_inner_interleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_symbol_inner_interleaver_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_symbol_inner_interleaver_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_symbol_inner_interleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_symbol_inner_interleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_symbol_inner_interleaver_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_symbol_inner_interleaver_sptr_message_subscribers(self, which_port)

# Register dvbt_symbol_inner_interleaver_sptr in _dtv_swig2:
_dtv_swig2.dvbt_symbol_inner_interleaver_sptr_swigregister(dvbt_symbol_inner_interleaver_sptr)


dvbt_symbol_inner_interleaver_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_symbol_inner_interleaver = dvbt_symbol_inner_interleaver.make;

class dvbt_map_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_map > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_map_sptr self) -> dvbt_map_sptr
        __init__(dvbt_map_sptr self, dvbt_map p) -> dvbt_map_sptr
        """
        _dtv_swig2.dvbt_map_sptr_swiginit(self, _dtv_swig2.new_dvbt_map_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_map *":
        r"""__deref__(dvbt_map_sptr self) -> dvbt_map"""
        return _dtv_swig2.dvbt_map_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_map_sptr

    def make(self, nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t", gain: "float") -> "gr::dtv::dvbt_map::sptr":
        r"""make(dvbt_map_sptr self, int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission, float gain) -> dvbt_map_sptr"""
        return _dtv_swig2.dvbt_map_sptr_make(self, nsize, constellation, hierarchy, transmission, gain)

    def history(self) -> "unsigned int":
        r"""history(dvbt_map_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_map_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_map_sptr self, int which, int delay)
        declare_sample_delay(dvbt_map_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_map_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_map_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_map_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_map_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_map_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_map_sptr self) -> int"""
        return _dtv_swig2.dvbt_map_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_map_sptr self) -> double"""
        return _dtv_swig2.dvbt_map_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_map_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_map_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_map_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_map_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_map_sptr self) -> bool"""
        return _dtv_swig2.dvbt_map_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_map_sptr self) -> bool"""
        return _dtv_swig2.dvbt_map_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_map_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_map_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_map_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_map_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_map_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_map_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_map_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_map_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_map_sptr self) -> int"""
        return _dtv_swig2.dvbt_map_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_map_sptr self, int m)"""
        return _dtv_swig2.dvbt_map_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_map_sptr self)"""
        return _dtv_swig2.dvbt_map_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_map_sptr self) -> bool"""
        return _dtv_swig2.dvbt_map_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_map_sptr self, int m)"""
        return _dtv_swig2.dvbt_map_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_map_sptr self) -> int"""
        return _dtv_swig2.dvbt_map_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_map_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_map_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_map_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_map_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_map_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_map_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_map_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_map_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_map_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_map_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_map_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_map_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_map_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_map_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_map_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_map_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_map_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_map_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_map_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_map_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_map_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_map_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_map_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_map_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_map_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_map_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_map_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_map_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_map_sptr self) -> float"""
        return _dtv_swig2.dvbt_map_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_map_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_map_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_map_sptr self)"""
        return _dtv_swig2.dvbt_map_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_map_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_map_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_map_sptr self) -> int"""
        return _dtv_swig2.dvbt_map_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_map_sptr self) -> int"""
        return _dtv_swig2.dvbt_map_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_map_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_map_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_map_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_map_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_map_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_map_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_map_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_map_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_map_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_map_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_map_sptr self) -> long"""
        return _dtv_swig2.dvbt_map_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_map_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_map_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_map_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_map_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_map_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_map_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_map_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_map_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_map_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_map_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_map_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_map_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_map_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_map_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_map_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_map_sptr_message_subscribers(self, which_port)

# Register dvbt_map_sptr in _dtv_swig2:
_dtv_swig2.dvbt_map_sptr_swigregister(dvbt_map_sptr)


dvbt_map_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_map = dvbt_map.make;

class dvbt_reference_signals_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_reference_signals > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_reference_signals_sptr self) -> dvbt_reference_signals_sptr
        __init__(dvbt_reference_signals_sptr self, dvbt_reference_signals p) -> dvbt_reference_signals_sptr
        """
        _dtv_swig2.dvbt_reference_signals_sptr_swiginit(self, _dtv_swig2.new_dvbt_reference_signals_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_reference_signals *":
        r"""__deref__(dvbt_reference_signals_sptr self) -> dvbt_reference_signals"""
        return _dtv_swig2.dvbt_reference_signals_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_reference_signals_sptr

    def make(self, itemsize: "int", ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", code_rate_HP: "gr::dtv::dvb_code_rate_t", code_rate_LP: "gr::dtv::dvb_code_rate_t", guard_interval: "gr::dtv::dvb_guardinterval_t", transmission_mode: "gr::dtv::dvbt_transmission_mode_t", include_cell_id: "int", cell_id: "int") -> "gr::dtv::dvbt_reference_signals::sptr":
        r"""make(dvbt_reference_signals_sptr self, int itemsize, int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t code_rate_HP, gr::dtv::dvb_code_rate_t code_rate_LP, gr::dtv::dvb_guardinterval_t guard_interval, gr::dtv::dvbt_transmission_mode_t transmission_mode, int include_cell_id, int cell_id) -> dvbt_reference_signals_sptr"""
        return _dtv_swig2.dvbt_reference_signals_sptr_make(self, itemsize, ninput, noutput, constellation, hierarchy, code_rate_HP, code_rate_LP, guard_interval, transmission_mode, include_cell_id, cell_id)

    def history(self) -> "unsigned int":
        r"""history(dvbt_reference_signals_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_reference_signals_sptr self, int which, int delay)
        declare_sample_delay(dvbt_reference_signals_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_reference_signals_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_reference_signals_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_reference_signals_sptr self) -> double"""
        return _dtv_swig2.dvbt_reference_signals_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_reference_signals_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_reference_signals_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_reference_signals_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_reference_signals_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_reference_signals_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reference_signals_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_reference_signals_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reference_signals_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_reference_signals_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_reference_signals_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_reference_signals_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_reference_signals_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_reference_signals_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reference_signals_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_reference_signals_sptr self, int m)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_reference_signals_sptr self)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_reference_signals_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reference_signals_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_reference_signals_sptr self, int m)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_reference_signals_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_reference_signals_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_reference_signals_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_reference_signals_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_reference_signals_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_reference_signals_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_reference_signals_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_reference_signals_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_reference_signals_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_reference_signals_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_reference_signals_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_reference_signals_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_reference_signals_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_reference_signals_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_reference_signals_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_reference_signals_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_reference_signals_sptr self)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_reference_signals_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_reference_signals_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_reference_signals_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_reference_signals_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reference_signals_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reference_signals_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_reference_signals_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_reference_signals_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_reference_signals_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_reference_signals_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_reference_signals_sptr self) -> long"""
        return _dtv_swig2.dvbt_reference_signals_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_reference_signals_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_reference_signals_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_reference_signals_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_reference_signals_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reference_signals_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_reference_signals_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_reference_signals_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_reference_signals_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_reference_signals_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_reference_signals_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reference_signals_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_reference_signals_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reference_signals_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_reference_signals_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reference_signals_sptr_message_subscribers(self, which_port)

# Register dvbt_reference_signals_sptr in _dtv_swig2:
_dtv_swig2.dvbt_reference_signals_sptr_swigregister(dvbt_reference_signals_sptr)


dvbt_reference_signals_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_reference_signals = dvbt_reference_signals.make;

class dvbt_ofdm_sym_acquisition_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_ofdm_sym_acquisition > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_ofdm_sym_acquisition_sptr self) -> dvbt_ofdm_sym_acquisition_sptr
        __init__(dvbt_ofdm_sym_acquisition_sptr self, dvbt_ofdm_sym_acquisition p) -> dvbt_ofdm_sym_acquisition_sptr
        """
        _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_swiginit(self, _dtv_swig2.new_dvbt_ofdm_sym_acquisition_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_ofdm_sym_acquisition *":
        r"""__deref__(dvbt_ofdm_sym_acquisition_sptr self) -> dvbt_ofdm_sym_acquisition"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_ofdm_sym_acquisition_sptr

    def make(self, blocks: "int", fft_length: "int", occupied_tones: "int", cp_length: "int", snr: "float") -> "gr::dtv::dvbt_ofdm_sym_acquisition::sptr":
        r"""make(dvbt_ofdm_sym_acquisition_sptr self, int blocks, int fft_length, int occupied_tones, int cp_length, float snr) -> dvbt_ofdm_sym_acquisition_sptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_make(self, blocks, fft_length, occupied_tones, cp_length, snr)

    def history(self) -> "unsigned int":
        r"""history(dvbt_ofdm_sym_acquisition_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_ofdm_sym_acquisition_sptr self, int which, int delay)
        declare_sample_delay(dvbt_ofdm_sym_acquisition_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_ofdm_sym_acquisition_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_ofdm_sym_acquisition_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_ofdm_sym_acquisition_sptr self) -> int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_ofdm_sym_acquisition_sptr self) -> double"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_ofdm_sym_acquisition_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_ofdm_sym_acquisition_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_ofdm_sym_acquisition_sptr self) -> bool"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_ofdm_sym_acquisition_sptr self) -> bool"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_ofdm_sym_acquisition_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_ofdm_sym_acquisition_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_ofdm_sym_acquisition_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_ofdm_sym_acquisition_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_ofdm_sym_acquisition_sptr self) -> int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_ofdm_sym_acquisition_sptr self, int m)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_ofdm_sym_acquisition_sptr self)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_ofdm_sym_acquisition_sptr self) -> bool"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_ofdm_sym_acquisition_sptr self, int m)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_ofdm_sym_acquisition_sptr self) -> int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_ofdm_sym_acquisition_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_ofdm_sym_acquisition_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_ofdm_sym_acquisition_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_ofdm_sym_acquisition_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_ofdm_sym_acquisition_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_ofdm_sym_acquisition_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_ofdm_sym_acquisition_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_ofdm_sym_acquisition_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_ofdm_sym_acquisition_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_ofdm_sym_acquisition_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_ofdm_sym_acquisition_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_ofdm_sym_acquisition_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_ofdm_sym_acquisition_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_ofdm_sym_acquisition_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_ofdm_sym_acquisition_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_ofdm_sym_acquisition_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_ofdm_sym_acquisition_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_ofdm_sym_acquisition_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_ofdm_sym_acquisition_sptr self) -> float"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_ofdm_sym_acquisition_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_ofdm_sym_acquisition_sptr self)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_ofdm_sym_acquisition_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_ofdm_sym_acquisition_sptr self) -> int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_ofdm_sym_acquisition_sptr self) -> int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_ofdm_sym_acquisition_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_ofdm_sym_acquisition_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_ofdm_sym_acquisition_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_ofdm_sym_acquisition_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_ofdm_sym_acquisition_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_ofdm_sym_acquisition_sptr self) -> long"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_ofdm_sym_acquisition_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_ofdm_sym_acquisition_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_ofdm_sym_acquisition_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_ofdm_sym_acquisition_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_ofdm_sym_acquisition_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_ofdm_sym_acquisition_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_ofdm_sym_acquisition_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_ofdm_sym_acquisition_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_message_subscribers(self, which_port)

# Register dvbt_ofdm_sym_acquisition_sptr in _dtv_swig2:
_dtv_swig2.dvbt_ofdm_sym_acquisition_sptr_swigregister(dvbt_ofdm_sym_acquisition_sptr)


dvbt_ofdm_sym_acquisition_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_ofdm_sym_acquisition = dvbt_ofdm_sym_acquisition.make;

class dvbt_demod_reference_signals_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_demod_reference_signals > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_demod_reference_signals_sptr self) -> dvbt_demod_reference_signals_sptr
        __init__(dvbt_demod_reference_signals_sptr self, dvbt_demod_reference_signals p) -> dvbt_demod_reference_signals_sptr
        """
        _dtv_swig2.dvbt_demod_reference_signals_sptr_swiginit(self, _dtv_swig2.new_dvbt_demod_reference_signals_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_demod_reference_signals *":
        r"""__deref__(dvbt_demod_reference_signals_sptr self) -> dvbt_demod_reference_signals"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_demod_reference_signals_sptr

    def make(self, itemsize: "int", ninput: "int", noutput: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", code_rate_HP: "gr::dtv::dvb_code_rate_t", code_rate_LP: "gr::dtv::dvb_code_rate_t", guard_interval: "gr::dtv::dvb_guardinterval_t", transmission_mode: "gr::dtv::dvbt_transmission_mode_t", include_cell_id: "int", cell_id: "int") -> "gr::dtv::dvbt_demod_reference_signals::sptr":
        r"""make(dvbt_demod_reference_signals_sptr self, int itemsize, int ninput, int noutput, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t code_rate_HP, gr::dtv::dvb_code_rate_t code_rate_LP, gr::dtv::dvb_guardinterval_t guard_interval, gr::dtv::dvbt_transmission_mode_t transmission_mode, int include_cell_id, int cell_id) -> dvbt_demod_reference_signals_sptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_make(self, itemsize, ninput, noutput, constellation, hierarchy, code_rate_HP, code_rate_LP, guard_interval, transmission_mode, include_cell_id, cell_id)

    def history(self) -> "unsigned int":
        r"""history(dvbt_demod_reference_signals_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_demod_reference_signals_sptr self, int which, int delay)
        declare_sample_delay(dvbt_demod_reference_signals_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_demod_reference_signals_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_demod_reference_signals_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_demod_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_demod_reference_signals_sptr self) -> double"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_demod_reference_signals_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_demod_reference_signals_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_demod_reference_signals_sptr self) -> bool"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_demod_reference_signals_sptr self) -> bool"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_demod_reference_signals_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_demod_reference_signals_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_demod_reference_signals_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_demod_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_demod_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_demod_reference_signals_sptr self, int m)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_demod_reference_signals_sptr self)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_demod_reference_signals_sptr self) -> bool"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_demod_reference_signals_sptr self, int m)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_demod_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_demod_reference_signals_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_demod_reference_signals_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_demod_reference_signals_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_demod_reference_signals_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_demod_reference_signals_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_demod_reference_signals_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_demod_reference_signals_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_demod_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_demod_reference_signals_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_demod_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_demod_reference_signals_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_demod_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_demod_reference_signals_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_demod_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_demod_reference_signals_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_demod_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_demod_reference_signals_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_demod_reference_signals_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_demod_reference_signals_sptr self) -> float"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_demod_reference_signals_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_demod_reference_signals_sptr self)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_demod_reference_signals_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_demod_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_demod_reference_signals_sptr self) -> int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_demod_reference_signals_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_demod_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_demod_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_demod_reference_signals_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_demod_reference_signals_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_demod_reference_signals_sptr self) -> long"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_demod_reference_signals_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_demod_reference_signals_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_demod_reference_signals_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_demod_reference_signals_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_demod_reference_signals_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_demod_reference_signals_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_demod_reference_signals_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_demod_reference_signals_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_demod_reference_signals_sptr_message_subscribers(self, which_port)

# Register dvbt_demod_reference_signals_sptr in _dtv_swig2:
_dtv_swig2.dvbt_demod_reference_signals_sptr_swigregister(dvbt_demod_reference_signals_sptr)


dvbt_demod_reference_signals_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_demod_reference_signals = dvbt_demod_reference_signals.make;

class dvbt_demap_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_demap > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_demap_sptr self) -> dvbt_demap_sptr
        __init__(dvbt_demap_sptr self, dvbt_demap p) -> dvbt_demap_sptr
        """
        _dtv_swig2.dvbt_demap_sptr_swiginit(self, _dtv_swig2.new_dvbt_demap_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_demap *":
        r"""__deref__(dvbt_demap_sptr self) -> dvbt_demap"""
        return _dtv_swig2.dvbt_demap_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_demap_sptr

    def make(self, nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t", gain: "float") -> "gr::dtv::dvbt_demap::sptr":
        r"""make(dvbt_demap_sptr self, int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission, float gain) -> dvbt_demap_sptr"""
        return _dtv_swig2.dvbt_demap_sptr_make(self, nsize, constellation, hierarchy, transmission, gain)

    def history(self) -> "unsigned int":
        r"""history(dvbt_demap_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_demap_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_demap_sptr self, int which, int delay)
        declare_sample_delay(dvbt_demap_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_demap_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_demap_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_demap_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_demap_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_demap_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_demap_sptr self) -> int"""
        return _dtv_swig2.dvbt_demap_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_demap_sptr self) -> double"""
        return _dtv_swig2.dvbt_demap_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_demap_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_demap_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_demap_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_demap_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_demap_sptr self) -> bool"""
        return _dtv_swig2.dvbt_demap_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_demap_sptr self) -> bool"""
        return _dtv_swig2.dvbt_demap_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_demap_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_demap_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_demap_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_demap_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_demap_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_demap_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_demap_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demap_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_demap_sptr self) -> int"""
        return _dtv_swig2.dvbt_demap_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_demap_sptr self, int m)"""
        return _dtv_swig2.dvbt_demap_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_demap_sptr self)"""
        return _dtv_swig2.dvbt_demap_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_demap_sptr self) -> bool"""
        return _dtv_swig2.dvbt_demap_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_demap_sptr self, int m)"""
        return _dtv_swig2.dvbt_demap_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_demap_sptr self) -> int"""
        return _dtv_swig2.dvbt_demap_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_demap_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_demap_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_demap_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_demap_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_demap_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_demap_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_demap_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_demap_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_demap_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_demap_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_demap_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_demap_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demap_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_demap_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_demap_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demap_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_demap_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_demap_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demap_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_demap_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_demap_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demap_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_demap_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_demap_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demap_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_demap_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_demap_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_demap_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_demap_sptr self) -> float"""
        return _dtv_swig2.dvbt_demap_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_demap_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_demap_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_demap_sptr self)"""
        return _dtv_swig2.dvbt_demap_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_demap_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_demap_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_demap_sptr self) -> int"""
        return _dtv_swig2.dvbt_demap_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_demap_sptr self) -> int"""
        return _dtv_swig2.dvbt_demap_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_demap_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_demap_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_demap_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demap_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_demap_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demap_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_demap_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_demap_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_demap_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_demap_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_demap_sptr self) -> long"""
        return _dtv_swig2.dvbt_demap_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_demap_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_demap_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_demap_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_demap_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_demap_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_demap_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_demap_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_demap_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_demap_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_demap_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_demap_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_demap_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_demap_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_demap_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_demap_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_demap_sptr_message_subscribers(self, which_port)

# Register dvbt_demap_sptr in _dtv_swig2:
_dtv_swig2.dvbt_demap_sptr_swigregister(dvbt_demap_sptr)


dvbt_demap_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_demap = dvbt_demap.make;

class dvbt_bit_inner_deinterleaver_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_bit_inner_deinterleaver > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_bit_inner_deinterleaver_sptr self) -> dvbt_bit_inner_deinterleaver_sptr
        __init__(dvbt_bit_inner_deinterleaver_sptr self, dvbt_bit_inner_deinterleaver p) -> dvbt_bit_inner_deinterleaver_sptr
        """
        _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_swiginit(self, _dtv_swig2.new_dvbt_bit_inner_deinterleaver_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_bit_inner_deinterleaver *":
        r"""__deref__(dvbt_bit_inner_deinterleaver_sptr self) -> dvbt_bit_inner_deinterleaver"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_bit_inner_deinterleaver_sptr

    def make(self, nsize: "int", constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", transmission: "gr::dtv::dvbt_transmission_mode_t") -> "gr::dtv::dvbt_bit_inner_deinterleaver::sptr":
        r"""make(dvbt_bit_inner_deinterleaver_sptr self, int nsize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvbt_transmission_mode_t transmission) -> dvbt_bit_inner_deinterleaver_sptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_make(self, nsize, constellation, hierarchy, transmission)

    def history(self) -> "unsigned int":
        r"""history(dvbt_bit_inner_deinterleaver_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_bit_inner_deinterleaver_sptr self, int which, int delay)
        declare_sample_delay(dvbt_bit_inner_deinterleaver_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_bit_inner_deinterleaver_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_bit_inner_deinterleaver_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_bit_inner_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_bit_inner_deinterleaver_sptr self) -> double"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_bit_inner_deinterleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_bit_inner_deinterleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_bit_inner_deinterleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_bit_inner_deinterleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_bit_inner_deinterleaver_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_bit_inner_deinterleaver_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_bit_inner_deinterleaver_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_bit_inner_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_bit_inner_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_bit_inner_deinterleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_bit_inner_deinterleaver_sptr self)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_bit_inner_deinterleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_bit_inner_deinterleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_bit_inner_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_bit_inner_deinterleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_bit_inner_deinterleaver_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_bit_inner_deinterleaver_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_bit_inner_deinterleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_bit_inner_deinterleaver_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_bit_inner_deinterleaver_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_bit_inner_deinterleaver_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_bit_inner_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_bit_inner_deinterleaver_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_bit_inner_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_bit_inner_deinterleaver_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_bit_inner_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_bit_inner_deinterleaver_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_bit_inner_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_bit_inner_deinterleaver_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_bit_inner_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_bit_inner_deinterleaver_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_bit_inner_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_bit_inner_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_bit_inner_deinterleaver_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_bit_inner_deinterleaver_sptr self)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_bit_inner_deinterleaver_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_bit_inner_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_bit_inner_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_bit_inner_deinterleaver_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_bit_inner_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_bit_inner_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_bit_inner_deinterleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_bit_inner_deinterleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_bit_inner_deinterleaver_sptr self) -> long"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_bit_inner_deinterleaver_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_bit_inner_deinterleaver_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_bit_inner_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_bit_inner_deinterleaver_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_bit_inner_deinterleaver_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_bit_inner_deinterleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_bit_inner_deinterleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_bit_inner_deinterleaver_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_message_subscribers(self, which_port)

# Register dvbt_bit_inner_deinterleaver_sptr in _dtv_swig2:
_dtv_swig2.dvbt_bit_inner_deinterleaver_sptr_swigregister(dvbt_bit_inner_deinterleaver_sptr)


dvbt_bit_inner_deinterleaver_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_bit_inner_deinterleaver = dvbt_bit_inner_deinterleaver.make;

class dvbt_viterbi_decoder_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_viterbi_decoder > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_viterbi_decoder_sptr self) -> dvbt_viterbi_decoder_sptr
        __init__(dvbt_viterbi_decoder_sptr self, dvbt_viterbi_decoder p) -> dvbt_viterbi_decoder_sptr
        """
        _dtv_swig2.dvbt_viterbi_decoder_sptr_swiginit(self, _dtv_swig2.new_dvbt_viterbi_decoder_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_viterbi_decoder *":
        r"""__deref__(dvbt_viterbi_decoder_sptr self) -> dvbt_viterbi_decoder"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_viterbi_decoder_sptr

    def make(self, constellation: "gr::dtv::dvb_constellation_t", hierarchy: "gr::dtv::dvbt_hierarchy_t", coderate: "gr::dtv::dvb_code_rate_t", bsize: "int") -> "gr::dtv::dvbt_viterbi_decoder::sptr":
        r"""make(dvbt_viterbi_decoder_sptr self, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt_hierarchy_t hierarchy, gr::dtv::dvb_code_rate_t coderate, int bsize) -> dvbt_viterbi_decoder_sptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_make(self, constellation, hierarchy, coderate, bsize)

    def history(self) -> "unsigned int":
        r"""history(dvbt_viterbi_decoder_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_viterbi_decoder_sptr self, int which, int delay)
        declare_sample_delay(dvbt_viterbi_decoder_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_viterbi_decoder_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_viterbi_decoder_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_viterbi_decoder_sptr self) -> int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_viterbi_decoder_sptr self) -> double"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_viterbi_decoder_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_viterbi_decoder_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_viterbi_decoder_sptr self) -> bool"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_viterbi_decoder_sptr self) -> bool"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_viterbi_decoder_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_viterbi_decoder_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_viterbi_decoder_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_viterbi_decoder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_viterbi_decoder_sptr self) -> int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_viterbi_decoder_sptr self, int m)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_viterbi_decoder_sptr self)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_viterbi_decoder_sptr self) -> bool"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_viterbi_decoder_sptr self, int m)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_viterbi_decoder_sptr self) -> int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_viterbi_decoder_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_viterbi_decoder_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_viterbi_decoder_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_viterbi_decoder_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_viterbi_decoder_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_viterbi_decoder_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_viterbi_decoder_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_viterbi_decoder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_viterbi_decoder_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_viterbi_decoder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_viterbi_decoder_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_viterbi_decoder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_viterbi_decoder_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_viterbi_decoder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_viterbi_decoder_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_viterbi_decoder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_viterbi_decoder_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_viterbi_decoder_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_viterbi_decoder_sptr self) -> float"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_viterbi_decoder_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_viterbi_decoder_sptr self)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_viterbi_decoder_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_viterbi_decoder_sptr self) -> int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_viterbi_decoder_sptr self) -> int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_viterbi_decoder_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_viterbi_decoder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_viterbi_decoder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_viterbi_decoder_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_viterbi_decoder_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_viterbi_decoder_sptr self) -> long"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_viterbi_decoder_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_viterbi_decoder_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_viterbi_decoder_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_viterbi_decoder_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_viterbi_decoder_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_viterbi_decoder_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_viterbi_decoder_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_viterbi_decoder_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_viterbi_decoder_sptr_message_subscribers(self, which_port)

# Register dvbt_viterbi_decoder_sptr in _dtv_swig2:
_dtv_swig2.dvbt_viterbi_decoder_sptr_swigregister(dvbt_viterbi_decoder_sptr)


dvbt_viterbi_decoder_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_viterbi_decoder = dvbt_viterbi_decoder.make;

class dvbt_convolutional_deinterleaver_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_convolutional_deinterleaver > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_convolutional_deinterleaver_sptr self) -> dvbt_convolutional_deinterleaver_sptr
        __init__(dvbt_convolutional_deinterleaver_sptr self, dvbt_convolutional_deinterleaver p) -> dvbt_convolutional_deinterleaver_sptr
        """
        _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_swiginit(self, _dtv_swig2.new_dvbt_convolutional_deinterleaver_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_convolutional_deinterleaver *":
        r"""__deref__(dvbt_convolutional_deinterleaver_sptr self) -> dvbt_convolutional_deinterleaver"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_convolutional_deinterleaver_sptr

    def make(self, nsize: "int", I: "int", M: "int") -> "gr::dtv::dvbt_convolutional_deinterleaver::sptr":
        r"""make(dvbt_convolutional_deinterleaver_sptr self, int nsize, int I, int M) -> dvbt_convolutional_deinterleaver_sptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_make(self, nsize, I, M)

    def history(self) -> "unsigned int":
        r"""history(dvbt_convolutional_deinterleaver_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_convolutional_deinterleaver_sptr self, int which, int delay)
        declare_sample_delay(dvbt_convolutional_deinterleaver_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_convolutional_deinterleaver_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_convolutional_deinterleaver_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_convolutional_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_convolutional_deinterleaver_sptr self) -> double"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_convolutional_deinterleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_convolutional_deinterleaver_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_convolutional_deinterleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_convolutional_deinterleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_convolutional_deinterleaver_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_convolutional_deinterleaver_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_convolutional_deinterleaver_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_convolutional_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_convolutional_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_convolutional_deinterleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_convolutional_deinterleaver_sptr self)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_convolutional_deinterleaver_sptr self) -> bool"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_convolutional_deinterleaver_sptr self, int m)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_convolutional_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_convolutional_deinterleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_convolutional_deinterleaver_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_convolutional_deinterleaver_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_convolutional_deinterleaver_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_convolutional_deinterleaver_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_convolutional_deinterleaver_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_convolutional_deinterleaver_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_convolutional_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_convolutional_deinterleaver_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_convolutional_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_convolutional_deinterleaver_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_convolutional_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_convolutional_deinterleaver_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_convolutional_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_convolutional_deinterleaver_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_convolutional_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_convolutional_deinterleaver_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_convolutional_deinterleaver_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_convolutional_deinterleaver_sptr self) -> float"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_convolutional_deinterleaver_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_convolutional_deinterleaver_sptr self)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_convolutional_deinterleaver_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_convolutional_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_convolutional_deinterleaver_sptr self) -> int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_convolutional_deinterleaver_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_convolutional_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_convolutional_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_convolutional_deinterleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_convolutional_deinterleaver_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_convolutional_deinterleaver_sptr self) -> long"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_convolutional_deinterleaver_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_convolutional_deinterleaver_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_convolutional_deinterleaver_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_convolutional_deinterleaver_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_convolutional_deinterleaver_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_convolutional_deinterleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_convolutional_deinterleaver_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_convolutional_deinterleaver_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_convolutional_deinterleaver_sptr_message_subscribers(self, which_port)

# Register dvbt_convolutional_deinterleaver_sptr in _dtv_swig2:
_dtv_swig2.dvbt_convolutional_deinterleaver_sptr_swigregister(dvbt_convolutional_deinterleaver_sptr)


dvbt_convolutional_deinterleaver_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_convolutional_deinterleaver = dvbt_convolutional_deinterleaver.make;

class dvbt_reed_solomon_dec_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_reed_solomon_dec > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_reed_solomon_dec_sptr self) -> dvbt_reed_solomon_dec_sptr
        __init__(dvbt_reed_solomon_dec_sptr self, dvbt_reed_solomon_dec p) -> dvbt_reed_solomon_dec_sptr
        """
        _dtv_swig2.dvbt_reed_solomon_dec_sptr_swiginit(self, _dtv_swig2.new_dvbt_reed_solomon_dec_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_reed_solomon_dec *":
        r"""__deref__(dvbt_reed_solomon_dec_sptr self) -> dvbt_reed_solomon_dec"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_reed_solomon_dec_sptr

    def make(self, p: "int", m: "int", gfpoly: "int", n: "int", k: "int", t: "int", s: "int", blocks: "int") -> "gr::dtv::dvbt_reed_solomon_dec::sptr":
        r"""make(dvbt_reed_solomon_dec_sptr self, int p, int m, int gfpoly, int n, int k, int t, int s, int blocks) -> dvbt_reed_solomon_dec_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_make(self, p, m, gfpoly, n, k, t, s, blocks)

    def history(self) -> "unsigned int":
        r"""history(dvbt_reed_solomon_dec_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_reed_solomon_dec_sptr self, int which, int delay)
        declare_sample_delay(dvbt_reed_solomon_dec_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_reed_solomon_dec_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_reed_solomon_dec_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_reed_solomon_dec_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_reed_solomon_dec_sptr self) -> double"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_reed_solomon_dec_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_reed_solomon_dec_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_reed_solomon_dec_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_reed_solomon_dec_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_reed_solomon_dec_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_reed_solomon_dec_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_reed_solomon_dec_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_reed_solomon_dec_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_reed_solomon_dec_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_reed_solomon_dec_sptr self, int m)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_reed_solomon_dec_sptr self)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_reed_solomon_dec_sptr self) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_reed_solomon_dec_sptr self, int m)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_reed_solomon_dec_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_reed_solomon_dec_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_reed_solomon_dec_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_reed_solomon_dec_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_reed_solomon_dec_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_reed_solomon_dec_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_reed_solomon_dec_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_reed_solomon_dec_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_reed_solomon_dec_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_reed_solomon_dec_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_reed_solomon_dec_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_reed_solomon_dec_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_reed_solomon_dec_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_reed_solomon_dec_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_reed_solomon_dec_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_reed_solomon_dec_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_reed_solomon_dec_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_reed_solomon_dec_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_reed_solomon_dec_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_reed_solomon_dec_sptr self) -> float"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_reed_solomon_dec_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_reed_solomon_dec_sptr self)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_reed_solomon_dec_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_reed_solomon_dec_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_reed_solomon_dec_sptr self) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_reed_solomon_dec_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_reed_solomon_dec_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_reed_solomon_dec_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_reed_solomon_dec_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_reed_solomon_dec_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_reed_solomon_dec_sptr self) -> long"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_reed_solomon_dec_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_reed_solomon_dec_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_reed_solomon_dec_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_reed_solomon_dec_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_reed_solomon_dec_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_reed_solomon_dec_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_reed_solomon_dec_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_reed_solomon_dec_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_reed_solomon_dec_sptr_message_subscribers(self, which_port)

# Register dvbt_reed_solomon_dec_sptr in _dtv_swig2:
_dtv_swig2.dvbt_reed_solomon_dec_sptr_swigregister(dvbt_reed_solomon_dec_sptr)


dvbt_reed_solomon_dec_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_reed_solomon_dec = dvbt_reed_solomon_dec.make;

class dvbt_energy_descramble_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt_energy_descramble > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt_energy_descramble_sptr self) -> dvbt_energy_descramble_sptr
        __init__(dvbt_energy_descramble_sptr self, dvbt_energy_descramble p) -> dvbt_energy_descramble_sptr
        """
        _dtv_swig2.dvbt_energy_descramble_sptr_swiginit(self, _dtv_swig2.new_dvbt_energy_descramble_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt_energy_descramble *":
        r"""__deref__(dvbt_energy_descramble_sptr self) -> dvbt_energy_descramble"""
        return _dtv_swig2.dvbt_energy_descramble_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig2.delete_dvbt_energy_descramble_sptr

    def make(self, nblocks: "int") -> "gr::dtv::dvbt_energy_descramble::sptr":
        r"""make(dvbt_energy_descramble_sptr self, int nblocks) -> dvbt_energy_descramble_sptr"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_make(self, nblocks)

    def history(self) -> "unsigned int":
        r"""history(dvbt_energy_descramble_sptr self) -> unsigned int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt_energy_descramble_sptr self, int which, int delay)
        declare_sample_delay(dvbt_energy_descramble_sptr self, unsigned int delay)
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt_energy_descramble_sptr self, int which) -> unsigned int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt_energy_descramble_sptr self, int multiple)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt_energy_descramble_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt_energy_descramble_sptr self) -> double"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt_energy_descramble_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt_energy_descramble_sptr self) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt_energy_descramble_sptr self) -> bool"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt_energy_descramble_sptr self) -> bool"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt_energy_descramble_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt_energy_descramble_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt_energy_descramble_sptr self, std::string level)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt_energy_descramble_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt_energy_descramble_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt_energy_descramble_sptr self, int m)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt_energy_descramble_sptr self)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt_energy_descramble_sptr self) -> bool"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt_energy_descramble_sptr self, int m)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt_energy_descramble_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt_energy_descramble_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt_energy_descramble_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt_energy_descramble_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt_energy_descramble_sptr self, int i) -> long"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt_energy_descramble_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt_energy_descramble_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt_energy_descramble_sptr self, int which) -> float
        pc_input_buffers_full(dvbt_energy_descramble_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt_energy_descramble_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt_energy_descramble_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt_energy_descramble_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt_energy_descramble_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt_energy_descramble_sptr self, int which) -> float
        pc_output_buffers_full(dvbt_energy_descramble_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt_energy_descramble_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt_energy_descramble_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt_energy_descramble_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt_energy_descramble_sptr self) -> pmt_vector_float
        """
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt_energy_descramble_sptr self) -> float"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt_energy_descramble_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt_energy_descramble_sptr self)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt_energy_descramble_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt_energy_descramble_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt_energy_descramble_sptr self) -> int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt_energy_descramble_sptr self, int priority) -> int"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt_energy_descramble_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt_energy_descramble_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt_energy_descramble_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt_energy_descramble_sptr self) -> io_signature_sptr"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt_energy_descramble_sptr self) -> long"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt_energy_descramble_sptr self) -> basic_block_sptr"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt_energy_descramble_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt_energy_descramble_sptr self) -> std::string"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt_energy_descramble_sptr self, std::string name)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt_energy_descramble_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig2.dvbt_energy_descramble_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt_energy_descramble_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt_energy_descramble_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt_energy_descramble_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig2.dvbt_energy_descramble_sptr_message_subscribers(self, which_port)

# Register dvbt_energy_descramble_sptr in _dtv_swig2:
_dtv_swig2.dvbt_energy_descramble_sptr_swigregister(dvbt_energy_descramble_sptr)


dvbt_energy_descramble_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt_energy_descramble = dvbt_energy_descramble.make;



